@@ -19,17 +19,19 @@
  * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  * Boston, MA  02111-1307  USA
  *
- * $Id: scim_setup_ui.cpp,v 1.24 2004/04/22 02:38:10 suzhe Exp $
+ * $Id: scim_setup_ui.cpp,v 1.42 2004/08/21 14:32:14 suzhe Exp $
  *
  */
 
 #include <cstring>
 #include <cstdio>
 
+#define Uses_SCIM_COMPOSE_KEY
 #define Uses_SCIM_CONFIG_BASE
 #define Uses_SCIM_CONFIG_PATH
 #define Uses_SCIM_MODULE
-#define Uses_SCIM_SERVER_MODULE
+#define Uses_SCIM_IMENGINE_MODULE
+#define Uses_STL_MAP
 #include "scim_private.h"
 #include "scim.h"
 #include "scim_setup_module.h"
@@ -41,7 +43,7 @@ const gchar * scim_setup_module_categories [] =
 {
     N_("SCIM"),
     N_("FrontEnd"),
-    N_("Server"),
+    N_("IMEngine"),
     N_("Panel"),
     N_("Extra"),
     NULL
@@ -58,11 +60,12 @@ enum
 
 enum
 {
-    SERVER_LIST_ENABLE = 0,
-    SERVER_LIST_ICON,
-    SERVER_LIST_NAME,
-    SERVER_LIST_UUID,
-    SERVER_LIST_NUM_COLUMNS
+    FACTORY_LIST_ENABLE = 0,
+    FACTORY_LIST_ICON,
+    FACTORY_LIST_NAME,
+    FACTORY_LIST_UUID,
+    FACTORY_LIST_INCONSISTENT,
+    FACTORY_LIST_NUM_COLUMNS
 };
 
 static GdkPixbuf *
@@ -98,7 +101,8 @@ SetupUI::SetupUI (int argc, char *argv [])
       m_query_changed_timeout (0),
       m_changes_applied (false)
 {
-    String def_config = "simple";
+    String def_config = scim_global_config_read (SCIM_GLOBAL_CONFIG_DEFAULT_CONFIG_MODULE, String ("simple"));
+
     int i = 0;
 
     while (i < argc) {
@@ -173,7 +177,7 @@ SetupUI::add_module (SetupModule *module)
                 gtk_tree_store_set (
                     m_module_list_model, &iter,
                     MODULE_LIST_LABEL,    module_label.c_str (),
-                    MODULE_LIST_CATEGORY, 0,
+                    MODULE_LIST_CATEGORY, NULL,
                     MODULE_LIST_MODULE,   module,
                     MODULE_LIST_WIDGET,   module_widget,
                     -1);
@@ -192,8 +196,8 @@ SetupUI::add_module (SetupModule *module)
 
     GtkWidget *cover;
 
-    if (module_category == String ("Server"))
-        cover = create_server_list_view ();
+    if (module_category == String ("IMEngine"))
+        cover = create_factory_list_view ();
     else
         cover = create_setup_cover (module_category.c_str ());
 
@@ -205,7 +209,7 @@ SetupUI::add_module (SetupModule *module)
         m_module_list_model, &parent,
         MODULE_LIST_LABEL,    _(module_category.c_str ()),
         MODULE_LIST_CATEGORY, module_category.c_str (),
-        MODULE_LIST_MODULE,   0,
+        MODULE_LIST_MODULE,   NULL,
         MODULE_LIST_WIDGET,   cover,
         -1);
 
@@ -213,7 +217,7 @@ SetupUI::add_module (SetupModule *module)
     gtk_tree_store_set (
         m_module_list_model, &iter,
         MODULE_LIST_LABEL,    module_label.c_str (),
-        MODULE_LIST_CATEGORY, 0,
+        MODULE_LIST_CATEGORY, NULL,
         MODULE_LIST_MODULE,   module,
         MODULE_LIST_WIDGET,   module_widget,
         -1);
@@ -223,7 +227,7 @@ SetupUI::add_module (SetupModule *module)
     return true;
 }
 
-void
+bool
 SetupUI::run ()
 {
     if (m_main_window) {
@@ -243,6 +247,7 @@ SetupUI::run ()
 
         gtk_main ();
     }
+    return m_changes_applied;
 }
 
 void
@@ -308,7 +313,7 @@ SetupUI::create_main_ui ()
     // Create module list column.
     module_list_cell = gtk_cell_renderer_text_new ();
     module_list_column = gtk_tree_view_column_new_with_attributes (
-                            NULL, module_list_cell, "text", 0, NULL);
+                            NULL, module_list_cell, "text", MODULE_LIST_LABEL, NULL);
 
     gtk_tree_view_append_column (GTK_TREE_VIEW (m_module_list_view), module_list_column);
 
@@ -506,7 +511,7 @@ SetupUI::create_module_list_model ()
     gtk_tree_store_set (m_module_list_model, &iter,
                         MODULE_LIST_LABEL,    _(scim_setup_module_categories [0]),
                         MODULE_LIST_CATEGORY, scim_setup_module_categories [0],
-                        MODULE_LIST_MODULE,   0,
+                        MODULE_LIST_MODULE,   NULL,
                         MODULE_LIST_WIDGET,   widget,
                         -1);
 
@@ -515,7 +520,7 @@ SetupUI::create_module_list_model ()
 }
 
 GtkWidget *
-SetupUI::create_server_list_view ()
+SetupUI::create_factory_list_view ()
 {
     GtkRequisition size;
 
@@ -531,7 +536,7 @@ SetupUI::create_server_list_view ()
 
     snprintf (buf, 127,
               _("<span size=\"x-large\">The Setup for %s modules.</span>"),
-              _("Server"));
+              _("IMEngine"));
 
     vbox = gtk_vbox_new (FALSE, 4);
     gtk_widget_show (vbox);
@@ -560,32 +565,15 @@ SetupUI::create_server_list_view ()
     gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolledwindow), 
                                          GTK_SHADOW_NONE);
  
-    m_server_list_view = gtk_tree_view_new ();
-    gtk_widget_show (m_server_list_view);
-    gtk_container_add (GTK_CONTAINER (scrolledwindow), m_server_list_view);
-    gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (m_server_list_view), TRUE);
-    gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (m_server_list_view), TRUE);
-
-    // Enable column
-    column = gtk_tree_view_column_new ();
-    gtk_tree_view_column_set_resizable (column, FALSE);
-    gtk_tree_view_column_set_title (column, _("Enable"));
-
-    renderer = gtk_cell_renderer_toggle_new ();
-    gtk_cell_renderer_toggle_set_radio (GTK_CELL_RENDERER_TOGGLE (renderer), FALSE);
-    gtk_tree_view_column_pack_start (column, renderer, FALSE);
-    gtk_tree_view_column_set_attributes (column, renderer,
-                                         "active", SERVER_LIST_ENABLE, NULL);
-
-    g_signal_connect (G_OBJECT (renderer), "toggled",
-                      G_CALLBACK (SetupUI::server_list_changed_callback),
-                      this);
-
-    gtk_tree_view_append_column (GTK_TREE_VIEW (m_server_list_view), column);
+    m_factory_list_view = gtk_tree_view_new ();
+    gtk_widget_show (m_factory_list_view);
+    gtk_container_add (GTK_CONTAINER (scrolledwindow), m_factory_list_view);
+    gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (m_factory_list_view), TRUE);
+    gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (m_factory_list_view), TRUE);
 
     // Name column
     column = gtk_tree_view_column_new ();
-    gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_GROW_ONLY);
+    gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
     gtk_tree_view_column_set_resizable (column, TRUE);
 
     gtk_tree_view_column_set_title (column, _("Name"));
@@ -593,21 +581,43 @@ SetupUI::create_server_list_view ()
     renderer = gtk_cell_renderer_pixbuf_new ();
     gtk_tree_view_column_pack_start (column, renderer, FALSE);
     gtk_tree_view_column_set_attributes (column, renderer,
-                                         "pixbuf", SERVER_LIST_ICON, NULL);
+                                         "pixbuf", FACTORY_LIST_ICON, NULL);
 
     renderer = gtk_cell_renderer_text_new ();
     gtk_tree_view_column_pack_start (column, renderer, TRUE);
     gtk_tree_view_column_set_attributes (column, renderer,
-                                         "text", SERVER_LIST_NAME, NULL);
+                                         "text", FACTORY_LIST_NAME, NULL);
 
-    gtk_tree_view_append_column (GTK_TREE_VIEW (m_server_list_view), column);
+    gtk_tree_view_append_column (GTK_TREE_VIEW (m_factory_list_view), column);
 
-    create_server_list_model ();
+    // Enable column
+    column = gtk_tree_view_column_new ();
+    gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_FIXED);
+    gtk_tree_view_column_set_resizable (column, FALSE);
+    gtk_tree_view_column_set_title (column, _("Enable"));
+
+    renderer = gtk_cell_renderer_toggle_new ();
+    gtk_cell_renderer_toggle_set_radio (GTK_CELL_RENDERER_TOGGLE (renderer), FALSE);
+    gtk_tree_view_column_pack_start (column, renderer, FALSE);
+    gtk_tree_view_column_set_attributes (column, renderer,
+                                         "active", FACTORY_LIST_ENABLE,
+                                         "inconsistent", FACTORY_LIST_INCONSISTENT,
+                                         NULL);
+
+    g_signal_connect (G_OBJECT (renderer), "toggled",
+                      G_CALLBACK (SetupUI::factory_list_changed_callback),
+                      this);
+
+    gtk_tree_view_append_column (GTK_TREE_VIEW (m_factory_list_view), column);
+
+    create_factory_list_model ();
+
+    gtk_tree_view_collapse_all (GTK_TREE_VIEW (m_factory_list_view));
 
     gtk_widget_size_request (vbox, &size);
 
-    if (size.width  < 320) size.width = 320;
-    if (size.height < 240) size.height = 240;
+    if (size.width  < 360) size.width = 360;
+    if (size.height < 300) size.height = 300;
 
     gtk_widget_set_size_request (vbox, size.width, size.height);
 
@@ -617,147 +627,209 @@ SetupUI::create_server_list_view ()
 }
 
 void
-SetupUI::create_server_list_model ()
+SetupUI::create_factory_list_model ()
 {
     std::vector <String> uuids;
     std::vector <String> names;
+    std::vector <String> langs;
     std::vector <String> icons;
-    GtkTreeIter iter;
-    GdkPixbuf   *pixbuf;
 
-    m_server_list_model = gtk_list_store_new (SERVER_LIST_NUM_COLUMNS,
+    std::map <String, std::vector <size_t> > groups;
+
+    GtkTreeIter iter;
+    GtkTreeIter parent;
+
+    String      lang_name;
+    GdkPixbuf   *pixbuf;
+    size_t i;
+
+    m_factory_list_model = gtk_tree_store_new (FACTORY_LIST_NUM_COLUMNS,
                                               G_TYPE_BOOLEAN,
                                               GDK_TYPE_PIXBUF,
                                               G_TYPE_STRING,
-                                              G_TYPE_STRING);
+                                              G_TYPE_STRING,
+                                              G_TYPE_BOOLEAN);
 
-    get_server_list (uuids, names, icons);
+    get_factory_list (uuids, names, langs, icons);
 
-    for (size_t i = 0; i < uuids.size (); ++i) {
-        pixbuf = gdk_pixbuf_new_from_file (icons [i].c_str (), NULL);
-        scale_pixbuf (&pixbuf, LIST_ICON_SIZE, LIST_ICON_SIZE);
-        gtk_list_store_append (m_server_list_model, &iter);
-        gtk_list_store_set (m_server_list_model, &iter,
-                            SERVER_LIST_ENABLE, true,
-                            SERVER_LIST_ICON, pixbuf,
-                            SERVER_LIST_NAME, names [i].c_str (),
-                            SERVER_LIST_UUID, uuids [i].c_str (),
-                            -1);
-        if (pixbuf)
-            g_object_unref (pixbuf);
+    for (i = 0; i < uuids.size (); ++i) {
+        groups [langs [i]].push_back (i);
     }
 
-    load_server_list_config ();
+    // Add language group
+    for (std::map <String, std::vector <size_t> >::iterator it = groups.begin ();
+         it != groups.end (); ++ it) {
+        lang_name = scim_get_language_name (it->first);
+        gtk_tree_store_append (m_factory_list_model, &parent, NULL);
+        gtk_tree_store_set (m_factory_list_model, &parent,
+                            FACTORY_LIST_ENABLE, true,
+                            FACTORY_LIST_ICON, NULL,
+                            FACTORY_LIST_NAME, lang_name.c_str (),
+                            FACTORY_LIST_UUID, NULL,
+                            FACTORY_LIST_INCONSISTENT, FALSE,
+                            -1);
 
-    gtk_tree_view_set_model (GTK_TREE_VIEW (m_server_list_view),
-                             GTK_TREE_MODEL (m_server_list_model));
+        // Add factories for this group
+        for (i = 0; i < it->second.size (); ++i) {
+            pixbuf = gdk_pixbuf_new_from_file (icons [it->second [i]].c_str (), NULL);
+            scale_pixbuf (&pixbuf, LIST_ICON_SIZE, LIST_ICON_SIZE);
+            gtk_tree_store_append (m_factory_list_model, &iter, &parent);
+            gtk_tree_store_set (m_factory_list_model, &iter,
+                                FACTORY_LIST_ENABLE, true,
+                                FACTORY_LIST_ICON, pixbuf,
+                                FACTORY_LIST_NAME, names [it->second [i]].c_str (),
+                                FACTORY_LIST_UUID, uuids [it->second [i]].c_str (),
+                                FACTORY_LIST_INCONSISTENT, FALSE,
+                                -1);
+            if (pixbuf)
+                g_object_unref (pixbuf);
+        }
+    }
+
+    load_factory_list_config ();
+
+    gtk_tree_view_set_model (GTK_TREE_VIEW (m_factory_list_view),
+                             GTK_TREE_MODEL (m_factory_list_model));
 }
 
 void
-SetupUI::get_server_list (std::vector <String> &uuids,
+SetupUI::get_factory_list (std::vector <String> &uuids,
                           std::vector <String> &names,
+                          std::vector <String> &langs,
                           std::vector <String> &icons)
 {
-    std::vector<String>   server_module_list;
-    ServerFactoryPointer  server;
-    ServerModule          server_module;
-    ConfigPointer         config;
+    std::vector<String>    module_list;
+    IMEngineFactoryPointer factory;
+    IMEngineModule         module;
+    ConfigPointer          config;
 
-    scim_get_server_module_list (server_module_list);
+    scim_get_imengine_module_list (module_list);
 
     uuids.clear ();
     names.clear ();
+    langs.clear ();
     icons.clear ();
 
     if (!m_config.null ()) config = m_config;
     else config = new DummyConfig ("scim");
 
-    for (size_t i = 0; i < server_module_list.size (); ++ i) {
-        server_module.load (server_module_list [i], config);
-        if (server_module.valid ()) {
-            for (size_t j = 0; j < server_module.number_of_servers (); ++j) {
+    // Add "English/European" factory first.
+    factory = new ComposeKeyFactory ();
+    uuids.push_back (factory->get_uuid ());
+    names.push_back (utf8_wcstombs (factory->get_name ()));
+    langs.push_back (scim_get_normalized_language (factory->get_language ()));
+    icons.push_back (factory->get_icon_file ());
+
+    for (size_t i = 0; i < module_list.size (); ++ i) {
+
+        module.load (module_list [i], config);
+
+        if (module.valid ()) {
+            for (size_t j = 0; j < module.number_of_factories (); ++j) {
                 try {
-                    server = server_module.create_factory (j);
+                    factory = module.create_factory (j);
                 } catch (...) {
-                    server.reset ();
+                    factory.reset ();
                 }
 
-                if (!server.null ()) {
-                    if (std::find (uuids.begin (), uuids.end (), server->get_uuid ()) == uuids.end ()) {
-                        uuids.push_back (server->get_uuid ());
-                        names.push_back (utf8_wcstombs (server->get_name ()));
-                        icons.push_back (server->get_icon_file ());
+                if (!factory.null ()) {
+                    if (std::find (uuids.begin (), uuids.end (), factory->get_uuid ()) == uuids.end ()) {
+                        uuids.push_back (factory->get_uuid ());
+                        names.push_back (utf8_wcstombs (factory->get_name ()));
+                        langs.push_back (scim_get_normalized_language (factory->get_language ()));
+                        icons.push_back (factory->get_icon_file ());
                     }
-                    server.reset ();
+                    factory.reset ();
                 }
             }
-            server_module.unload ();
+            module.unload ();
         }
     }
 }
 
-void
-SetupUI::load_server_list_config ()
+gboolean
+SetupUI::factory_list_load_config_func (GtkTreeModel *model,
+                                       GtkTreePath  *path,
+                                       GtkTreeIter  *iter,
+                                       gpointer      data)
 {
-    std::vector <String> disabled;
-    GtkTreeIter iter;
+    gchar *uuid;
+    std::vector <String> *disabled = static_cast <std::vector<String> *> (data);
 
-    if (!m_config.null () && m_server_list_model) {
-        m_config->read (String (SCIM_CONFIG_DISABLED_SERVER_FACTORIES), &disabled);
-
-        if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (m_server_list_model), &iter)) {
-            gchar *uuid;
-            do {
-                gtk_tree_model_get (GTK_TREE_MODEL (m_server_list_model), &iter,
-                                    SERVER_LIST_UUID, &uuid,
-                                    -1);
-                if (uuid &&
-                    std::find (disabled.begin (), disabled.end (), String (uuid)) != disabled.end ()) {
-                    gtk_list_store_set (m_server_list_model, &iter,
-                                        SERVER_LIST_ENABLE, FALSE,
-                                        -1);
-                } else {
-                    gtk_list_store_set (m_server_list_model, &iter,
-                                        SERVER_LIST_ENABLE, TRUE,
-                                        -1);
-                }
-
-                if (uuid) g_free (uuid);
-            } while (gtk_tree_model_iter_next (GTK_TREE_MODEL (m_server_list_model), &iter));
-        }
+    gtk_tree_model_get (model, iter,
+                        FACTORY_LIST_UUID, &uuid,
+                        -1);
+    if (uuid &&
+        std::find (disabled->begin (), disabled->end (), String (uuid)) != disabled->end ()) {
+        gtk_tree_store_set (GTK_TREE_STORE (model), iter,
+                            FACTORY_LIST_ENABLE, FALSE,
+                            -1);
+    } else {
+        gtk_tree_store_set (GTK_TREE_STORE (model), iter,
+                            FACTORY_LIST_ENABLE, TRUE,
+                            -1);
     }
 
-    m_server_list_changed = false;
+    if (uuid) g_free (uuid);
+
+    return FALSE;
 }
 
 void
-SetupUI::save_server_list_config ()
+SetupUI::load_factory_list_config ()
 {
     std::vector <String> disabled;
-    GtkTreeIter iter;
 
-    if (!m_config.null () && m_server_list_model) {
-        if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (m_server_list_model), &iter)) {
-            gchar *uuid;
-            gboolean enable;
+    if (m_factory_list_model) {
+        disabled = scim_global_config_read (String (SCIM_GLOBAL_CONFIG_DISABLED_IMENGINE_FACTORIES), disabled);
 
-            do {
-                gtk_tree_model_get (GTK_TREE_MODEL (m_server_list_model), &iter,
-                                    SERVER_LIST_UUID, &uuid,
-                                    SERVER_LIST_ENABLE, &enable,
-                                    -1);
-                if (!enable && uuid)
-                    disabled.push_back (String (uuid));
+        gtk_tree_model_foreach (GTK_TREE_MODEL (m_factory_list_model),
+                                factory_list_load_config_func,
+                                static_cast <gpointer> (&disabled));
 
-                if (uuid) g_free (uuid);
-            } while (gtk_tree_model_iter_next (GTK_TREE_MODEL (m_server_list_model), &iter));
-        }
-
-        m_config->write (String (SCIM_CONFIG_DISABLED_SERVER_FACTORIES), disabled);
-        m_changes_applied = true;
+        factory_list_update_inconsistent ();
     }
 
-    m_server_list_changed = false;
+    m_factory_list_changed = false;
+}
+
+gboolean
+SetupUI::factory_list_save_config_func (GtkTreeModel *model,
+                                       GtkTreePath  *path,
+                                       GtkTreeIter  *iter,
+                                       gpointer      data)
+{
+    gchar *uuid;
+    gboolean enable;
+    std::vector <String> *disabled = static_cast <std::vector<String> *> (data);
+
+    gtk_tree_model_get (model, iter,
+                        FACTORY_LIST_UUID, &uuid,
+                        FACTORY_LIST_ENABLE, &enable,
+                        -1);
+
+    if (!enable && uuid)
+    disabled->push_back (String (uuid));
+
+    if (uuid) g_free (uuid);
+
+    return FALSE;
+}
+
+void
+SetupUI::save_factory_list_config ()
+{
+    std::vector <String> disabled;
+
+    if (m_factory_list_model && m_factory_list_changed) {
+        gtk_tree_model_foreach (GTK_TREE_MODEL (m_factory_list_model),
+                                factory_list_save_config_func,
+                                static_cast <gpointer> (&disabled));
+
+        scim_global_config_write (String (SCIM_GLOBAL_CONFIG_DISABLED_IMENGINE_FACTORIES), disabled);
+    }
+
+    m_factory_list_changed = false;
 }
 
 bool
@@ -837,9 +909,9 @@ SetupUI::module_list_selection_changed_callback (GtkTreeSelection *selection, gp
                     gtk_widget_set_sensitive (ui->m_apply_button, TRUE);
                     gtk_widget_set_sensitive (ui->m_restore_button, TRUE);
                 }
-            } else if (category && String (category) == String ("Server")){
+            } else if (category && String (category) == String ("IMEngine")){
                 gtk_statusbar_push (GTK_STATUSBAR (ui->m_status_bar), 1, _("You can enable/disable installed input methods here."));
-                if (ui->m_server_list_changed) {
+                if (ui->m_factory_list_changed) {
                     gtk_widget_set_sensitive (ui->m_apply_button, TRUE);
                     gtk_widget_set_sensitive (ui->m_restore_button, TRUE);
                 }
@@ -878,8 +950,8 @@ SetupUI::restore_button_clicked_callback (GtkButton *button, gpointer user_data)
             gtk_tree_model_get (model, &iter,
                                 MODULE_LIST_CATEGORY, &category,
                                 -1);
-            if (category && String (category) == String ("Server")) {
-                ui->load_server_list_config ();
+            if (category && String (category) == String ("IMEngine")) {
+                ui->load_factory_list_config ();
                 gtk_widget_set_sensitive (ui->m_apply_button, FALSE);
                 gtk_widget_set_sensitive (ui->m_restore_button, FALSE);
             }
@@ -917,8 +989,8 @@ SetupUI::apply_button_clicked_callback (GtkButton *button, gpointer user_data)
             gtk_tree_model_get (model, &iter,
                                 MODULE_LIST_CATEGORY, &category,
                                 -1);
-            if (category && String (category) == String ("Server")) {
-                ui->save_server_list_config ();
+            if (category && String (category) == String ("IMEngine")) {
+                ui->save_factory_list_config ();
                 gtk_widget_set_sensitive (ui->m_apply_button, FALSE);
                 gtk_widget_set_sensitive (ui->m_restore_button, FALSE);
             }
@@ -928,26 +1000,79 @@ SetupUI::apply_button_clicked_callback (GtkButton *button, gpointer user_data)
     }
 }
 
+void 
+SetupUI::factory_list_update_inconsistent(void)
+{
+    GtkTreeIter iter;
+    GtkTreeIter childiter;
+    gboolean    enable;
+    gboolean    inconsistent;
+
+    if (!m_factory_list_model || !gtk_tree_model_get_iter_first (GTK_TREE_MODEL (m_factory_list_model), &iter))
+        return;
+
+    do {
+        gtk_tree_model_get (GTK_TREE_MODEL (m_factory_list_model), &iter,
+                            FACTORY_LIST_ENABLE, &enable,
+                            FACTORY_LIST_INCONSISTENT, &inconsistent,
+                            -1);
+        if (gtk_tree_model_iter_children (GTK_TREE_MODEL (m_factory_list_model), &childiter, &iter)) {
+            gint enable_count = 0;
+            gint total_count = gtk_tree_model_iter_n_children(GTK_TREE_MODEL (m_factory_list_model), &iter);
+            do {
+                gboolean child_enable;
+                gtk_tree_model_get (GTK_TREE_MODEL (m_factory_list_model), &childiter,
+                                    FACTORY_LIST_ENABLE, &child_enable,
+                                    -1);
+                if (child_enable) ++ enable_count;
+            } while (gtk_tree_model_iter_next (GTK_TREE_MODEL (m_factory_list_model), &childiter));
+            enable = (enable_count && enable_count >= ((total_count+1) >> 1));
+            inconsistent = (enable_count && enable_count < total_count);
+        }
+        gtk_tree_store_set (GTK_TREE_STORE (m_factory_list_model), &iter,
+                            FACTORY_LIST_ENABLE, enable,
+                            FACTORY_LIST_INCONSISTENT, inconsistent,
+                            -1);
+    } while (gtk_tree_model_iter_next (GTK_TREE_MODEL (m_factory_list_model), &iter));
+}
+
 void
-SetupUI::server_list_changed_callback (GtkCellRendererToggle *cell, gchar *arg1, gpointer data)
+SetupUI::factory_list_changed_callback (GtkCellRendererToggle *cell, gchar *arg1, gpointer data)
 {
     SetupUI *ui = (SetupUI*) data;
     GtkTreePath *path;
-    GtkTreeIter iter;
+    GtkTreeIter iter, childiter;
     gboolean enable;
 
     path = gtk_tree_path_new_from_string (arg1);
-    if (gtk_tree_model_get_iter (GTK_TREE_MODEL (ui->m_server_list_model), &iter, path)) {
-        gtk_tree_model_get (GTK_TREE_MODEL (ui->m_server_list_model), &iter,
-                            SERVER_LIST_ENABLE, &enable,
-                            -1);
-        gtk_list_store_set (ui->m_server_list_model, &iter,
-                            SERVER_LIST_ENABLE, !enable,
-                            -1);
+    if (gtk_tree_model_get_iter (GTK_TREE_MODEL (ui->m_factory_list_model), &iter, path)) {
+        if (gtk_tree_model_iter_children(GTK_TREE_MODEL (ui->m_factory_list_model), &childiter, &iter)) {
+            gtk_tree_model_get (GTK_TREE_MODEL (ui->m_factory_list_model), &iter,
+                                FACTORY_LIST_ENABLE, &enable,
+                                -1);
+            enable = !enable;
+            gtk_tree_store_set (ui->m_factory_list_model, &iter,
+                                FACTORY_LIST_ENABLE, enable,
+                                FACTORY_LIST_INCONSISTENT, FALSE,
+                                -1);
+            do {
+                gtk_tree_store_set (ui->m_factory_list_model, &childiter,
+                                    FACTORY_LIST_ENABLE, enable,
+                                    -1);
+            } while (gtk_tree_model_iter_next (GTK_TREE_MODEL (ui->m_factory_list_model), &childiter));
+        } else {
+            gtk_tree_model_get (GTK_TREE_MODEL (ui->m_factory_list_model), &iter,
+                                FACTORY_LIST_ENABLE, &enable,
+                                -1);
+            gtk_tree_store_set (ui->m_factory_list_model, &iter,
+                                FACTORY_LIST_ENABLE, !enable,
+                                -1);
+            ui->factory_list_update_inconsistent ();
+        }
     }
     gtk_tree_path_free (path);
  
-    ui->m_server_list_changed = true;
+    ui->m_factory_list_changed = true;
 }
 
 gboolean
@@ -999,7 +1124,7 @@ SetupUI::ok_button_clicked_callback (GtkButton *button, gpointer user_data)
                                 module_list_save_config_iter_func,
                                 user_data); 
 
-        ui->save_server_list_config ();
+        ui->save_factory_list_config ();
 
         ui->m_config->flush ();
         if (ui->m_changes_applied)
@@ -1037,7 +1162,7 @@ SetupUI::config_changed_callback (GtkEditable *editable, gpointer user_data)
                             module_list_load_config_iter_func,
                             user_data); 
 
-            ui->load_server_list_config ();
+            ui->load_factory_list_config ();
 
             GtkWidget *dialog;
 
@@ -1099,7 +1224,7 @@ SetupUI::query_changed_timeout_cb (gpointer data)
             gtk_tree_model_get (model, &iter,
                                 MODULE_LIST_CATEGORY, &category,
                                 -1);
-            if (category && String (category) == String ("Server") && ui->m_server_list_changed) {
+            if (category && String (category) == String ("IMEngine") && ui->m_factory_list_changed) {
                 gtk_widget_set_sensitive (ui->m_apply_button, TRUE);
                 gtk_widget_set_sensitive (ui->m_restore_button, TRUE);
             }
@@ -1120,9 +1245,9 @@ SetupUI::show_restart_hint () const
                             GTK_DIALOG_MODAL,
                             GTK_MESSAGE_INFO,
                             GTK_BUTTONS_OK,
-                            _("Don't forget to restart SCIM "
-                              "in order to let the new "
-                              "configurations take effect."));
+                            _("Not all configuration can be reloaded on the fly. "
+                              "Don't forget to restart SCIM in order to let all of "
+                              "the new configuration take effect."));
 
     gtk_dialog_run (GTK_DIALOG (dialog));
 
