@@ -23,16 +23,25 @@
  * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  * Boston, MA  02111-1307  USA
  *
- * $Id: scim_private.cpp,v 1.1.2.1 2004/05/01 13:23:48 suzhe Exp $
+ * $Id: scim_private.cpp,v 1.4 2004/07/10 02:31:30 suzhe Exp $
  *
  */
 
 #include "scim_private.h"
+#include "scim.h"
 
 #ifdef HAVE_LOCALE_H
   #include <locale.h>
 #endif
 
+#include <sys/time.h>
+
+#ifdef TIME_WITH_SYS_TIME
+  #include <time.h>
+#endif
+
+#include <stdlib.h>
+
 namespace scim {
 
 class TextdomainInitializer {
@@ -48,7 +57,22 @@ public:
     }
 };
 
+class RandSeedInitializer {
+public:
+    RandSeedInitializer () {
+#ifdef HAVE_GETTIMEOFDAY
+        timeval cur_time;
+        if (gettimeofday (&cur_time, 0) == 0) {
+            srand (cur_time.tv_sec);
+        }
+#else
+        srand (time (0));
+#endif
+    }
+};
+
 static TextdomainInitializer __textdomain_initializer;
+static RandSeedInitializer   __rand_seed_initializer;
 
 } // namespace scim
 
