@@ -37,7 +37,7 @@
  * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  * Boston, MA  02111-1307  USA
  *
- * $Id: scim_signals.h,v 1.7 2004/02/06 07:53:15 suzhe Exp $
+ * $Id: scim_signals.h,v 1.9 2004/05/21 15:18:29 suzhe Exp $
  */
 
 #ifndef __SCIM_SIGNALS_H
@@ -112,7 +112,7 @@ class Signal
     Signal& operator=(const Signal&);
 
 protected:
-    typedef std::vector<SlotNode*> ConnectionList;
+    typedef std::vector< Pointer<SlotNode> > ConnectionList;
     //!< ConnectionList type.
 
     ConnectionList connection_list;
@@ -176,7 +176,7 @@ public:
             if (!(*i)->blocked())
             {
                 SlotType *slot = dynamic_cast<SlotType*>((*i)->slot());
-                if (m.marshal(slot->call()))
+                if (slot && m.marshal(slot->call()))
                     break;
             }
             ++i;
@@ -229,7 +229,7 @@ public:
             if (!(*i)->blocked())
             {
                 SlotType *slot = dynamic_cast<SlotType*>((*i)->slot());
-                slot->call();
+                if (slot) slot->call();
             }
             ++i;
         }
@@ -288,7 +288,7 @@ public:
             if (!(*i)->blocked())
             {
                 SlotType *slot = dynamic_cast<SlotType*>((*i)->slot());
-                if (m.marshal(slot->call(p1)))
+                if (slot && m.marshal(slot->call(p1)))
                     break;
             }
             ++i;        
@@ -342,7 +342,7 @@ public:
             if (!(*i)->blocked())
             {
                 SlotType *slot = dynamic_cast<SlotType*>((*i)->slot());
-                slot->call(p1);
+                if (slot) slot->call(p1);
             }
             ++i;
         }
@@ -402,7 +402,7 @@ public:
             if (!(*i)->blocked())
             {
                 SlotType *slot = dynamic_cast<SlotType*>((*i)->slot());
-                if (m.marshal(slot->call(p1, p2)))
+                if (slot && m.marshal(slot->call(p1, p2)))
                     break;
             }
             ++i;        
@@ -457,7 +457,7 @@ public:
             if (!(*i)->blocked())
             {
                 SlotType *slot = dynamic_cast<SlotType*>((*i)->slot());
-                slot->call(p1, p2);
+                if (slot) slot->call(p1, p2);
             }
             ++i;
         }
@@ -517,7 +517,7 @@ public:
             if (!(*i)->blocked())
             {
                 SlotType *slot = dynamic_cast<SlotType*>((*i)->slot());
-                if (m.marshal(slot->call(p1, p2, p3)))
+                if (slot && m.marshal(slot->call(p1, p2, p3)))
                     break;
             }
             ++i;        
@@ -573,7 +573,7 @@ public:
             if (!(*i)->blocked())
             {
                 SlotType *slot = dynamic_cast<SlotType*>((*i)->slot());
-                slot->call(p1, p2, p3);
+                if (slot) slot->call(p1, p2, p3);
             }
             ++i;
         }
@@ -633,7 +633,7 @@ public:
             if (!(*i)->blocked())
             {
                 SlotType *slot = dynamic_cast<SlotType*>((*i)->slot());
-                if (m.marshal(slot->call(p1, p2, p3, p4)))
+                if (slot && m.marshal(slot->call(p1, p2, p3, p4)))
                     break;
             }
             ++i;        
@@ -690,7 +690,7 @@ public:
             if (!(*i)->blocked())
             {
                 SlotType *slot = dynamic_cast<SlotType*>((*i)->slot());
-                slot->call(p1, p2, p3, p4);
+                if (slot) slot->call(p1, p2, p3, p4);
             }
             ++i;
         }
@@ -750,7 +750,7 @@ public:
             if (!(*i)->blocked())
             {
                 SlotType *slot = dynamic_cast<SlotType*>((*i)->slot());
-                if (m.marshal(slot->call(p1, p2, p3, p4, p5)))
+                if (slot && m.marshal(slot->call(p1, p2, p3, p4, p5)))
                     break;
             }
             ++i;        
@@ -808,7 +808,7 @@ public:
             if (!(*i)->blocked())
             {
                 SlotType *slot = dynamic_cast<SlotType*>((*i)->slot());
-                slot->call(p1, p2, p3, p4, p5);
+                if (slot) slot->call(p1, p2, p3, p4, p5);
             }
             ++i;
         }
@@ -868,7 +868,7 @@ public:
             if (!(*i)->blocked())
             {
                 SlotType *slot = dynamic_cast<SlotType*>((*i)->slot());
-                if (m.marshal(slot->call(p1, p2, p3, p4, p5, p6)))
+                if (slot && m.marshal(slot->call(p1, p2, p3, p4, p5, p6)))
                     break;
             }
             ++i;        
@@ -928,7 +928,7 @@ public:
             if (!(*i)->blocked())
             {
                 SlotType *slot = dynamic_cast<SlotType*>((*i)->slot());
-                slot->call(p1, p2, p3, p4, p5, p6);
+                if (slot) slot->call(p1, p2, p3, p4, p5, p6);
             }
             ++i;
         }
