@@ -23,11 +23,12 @@
  * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  * Boston, MA  02111-1307  USA
  *
- * $Id: scim_socket.cpp,v 1.25 2004/01/14 09:31:54 suzhe Exp $
+ * $Id: scim_socket.cpp,v 1.31.2.1 2004/11/08 01:57:49 suzhe Exp $
  *
  */
 
 #define Uses_SCIM_SOCKET
+#define Uses_SCIM_CONFIG_PATH
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <sys/stat.h>
@@ -183,7 +184,7 @@ SocketAddress::SocketAddressImpl::set_address (const String &addr)
 
         struct sockaddr_un *un = new struct sockaddr_un;
 
-        un->sun_family = AF_LOCAL;
+        un->sun_family = AF_UNIX;
 
         memset (un->sun_path, 0, sizeof (un->sun_path));
 
@@ -335,8 +336,9 @@ public:
         if (timeout < 0)
             return read (buf, size);
  
-        int ret;
-        int nbytes = 0;
+        int   ret;
+        int   nbytes = 0;
+        char *cbuf = static_cast<char *> (buf);
  
         while (size > 0) {
             ret = wait_for_data (timeout);
@@ -344,12 +346,12 @@ public:
             if (ret < 0) return ret;
             if (ret == 0) return nbytes;
  
-            ret = read (buf, size);
+            ret = read (cbuf, size);
  
             if (ret < 0) return ret;
             if (ret == 0) return nbytes;
  
-            ((char*)buf) += ret;
+            cbuf += ret;
             nbytes += ret;
             size -= ret;
         }
@@ -367,11 +369,13 @@ public:
  
         m_err = 0;
 
+        const char *cbuf = static_cast<const char*> (buf);
+
         while (size > 0) {
-            ret = ::write (m_id, buf, size);
+            ret = ::write (m_id, cbuf, size);
             if (ret <= 0) break;
             size -= (size_t) ret;
-            ((char*)buf) += ret;
+            cbuf += ret;
         }
  
         if (ret == 0) m_err = EPIPE;
@@ -545,7 +549,7 @@ public:
         int ret = -1;
  
         if (family == SCIM_SOCKET_LOCAL) 
-            ret = ::socket (PF_LOCAL, SOCK_STREAM, 0);
+            ret = ::socket (PF_UNIX, SOCK_STREAM, 0);
         else if (family == SCIM_SOCKET_INET)
             ret = ::socket (PF_INET, SOCK_STREAM, 0);
         else {
@@ -822,16 +826,18 @@ SocketServer::run ()
                             if (m_impl->max_fd < client)
                                 m_impl->max_fd = client;
 
+                            Socket client_socket (client);
                             //emit the signal.
-                            m_impl->accept_signal.emit (this, Socket (client));
+                            m_impl->accept_signal.emit (this, client_socket);
                         }
 
                     //Client reading
                     } else {
-
                         SCIM_DEBUG_SOCKET (3) << "  SocketServer: Accept client reading...\n";
 
-                        m_impl->receive_signal.emit (this, Socket (i));
+                        Socket client_socket (i);
+                        //emit the signal.
+                        m_impl->receive_signal.emit (this, client_socket);
                     }
                 }
 
@@ -846,12 +852,12 @@ SocketServer::run ()
                         return true;
 
                     } else {
-
                         SCIM_DEBUG_SOCKET (3) << "  SocketServer: Client "
                                               << i
                                               << "got an exception, callbacking...\n";
 
-                        m_impl->exception_signal.emit (this, Socket (i));
+                        Socket client_socket (i);
+                        m_impl->exception_signal.emit (this, client_socket);
                     }
                 }
 
@@ -905,14 +911,12 @@ void
 SocketServer::close_connection (const Socket &socket)
 {
     int id = socket.get_id ();
-    if (m_impl->created && m_impl->running && id > 0) {
+    if (m_impl->created && m_impl->running && id > 0 && FD_ISSET (id, &(m_impl->active_fds))) {
 
         SCIM_DEBUG_SOCKET (2) << " SocketServer: Closing the connection: "
                               << socket.get_id () << "\n";
 
-        for (int i=0; i<m_impl->max_fd+1; i++)
-            if (FD_ISSET (id, &(m_impl->active_fds)))
-                m_impl->num_clients --;
+        m_impl->num_clients --;
 
         FD_CLR (id, &(m_impl->active_fds));
 
@@ -1013,6 +1017,106 @@ SocketClient::close ()
     m_connected = false;
 }
 
+#define SCIM_DEFAULT_SOCKET_FRONTEND_ADDRESS    "local:/tmp/scim-socket-frontend"
+#define SCIM_DEFAULT_PANEL_SOCKET_ADDRESS       "local:/tmp/scim-panel-socket"
+
+String scim_get_default_socket_frontend_address ()
+{
+    String address (SCIM_DEFAULT_SOCKET_FRONTEND_ADDRESS);
+
+    address = scim_global_config_read (SCIM_GLOBAL_CONFIG_DEFAULT_SOCKET_FRONTEND_ADDRESS, address);
+
+    const char *env = getenv ("SCIM_SOCKET_ADDRESS");
+    if (env && strlen (env) > 0) {
+        address = String (env);
+    } else {
+        env = getenv ("SCIM_FRONTEND_SOCKET_ADDRESS");
+        if (env && strlen (env))
+            address = String (env);
+    }
+
+    if (address == "default")
+        address = SCIM_DEFAULT_SOCKET_FRONTEND_ADDRESS;
+
+    return address;
+}
+
+String scim_get_default_socket_imengine_address ()
+{
+    String address (SCIM_DEFAULT_SOCKET_FRONTEND_ADDRESS);
+
+    address = scim_global_config_read (SCIM_GLOBAL_CONFIG_DEFAULT_SOCKET_IMENGINE_ADDRESS, address);
+
+    const char *env = getenv ("SCIM_SOCKET_ADDRESS");
+    if (env && strlen (env) > 0) {
+        address = String (env);
+    } else {
+        env = getenv ("SCIM_IMENGINE_SOCKET_ADDRESS");
+        if (env && strlen (env))
+            address = String (env);
+    }
+
+    if (address == "default")
+        address = SCIM_DEFAULT_SOCKET_FRONTEND_ADDRESS;
+
+    return address;
+}
+
+String scim_get_default_socket_config_address ()
+{
+    String address (SCIM_DEFAULT_SOCKET_FRONTEND_ADDRESS);
+
+    address = scim_global_config_read (SCIM_GLOBAL_CONFIG_DEFAULT_SOCKET_CONFIG_ADDRESS, address);
+
+    const char *env = getenv ("SCIM_SOCKET_ADDRESS");
+    if (env && strlen (env) > 0) {
+        address = String (env);
+    } else {
+        env = getenv ("SCIM_CONFIG_SOCKET_ADDRESS");
+        if (env && strlen (env))
+            address = String (env);
+    }
+
+    if (address == "default")
+        address = SCIM_DEFAULT_SOCKET_FRONTEND_ADDRESS;
+
+    return address;
+}
+
+String scim_get_default_panel_socket_address ()
+{
+    String address (SCIM_DEFAULT_PANEL_SOCKET_ADDRESS);
+
+    address = scim_global_config_read (SCIM_GLOBAL_CONFIG_DEFAULT_PANEL_SOCKET_ADDRESS, address);
+
+    const char *env = getenv ("SCIM_PANEL_SOCKET_ADDRESS");
+
+    if (env && strlen (env) > 0) {
+        address = String (env);
+    }
+
+    if (address == "default")
+        address = SCIM_DEFAULT_PANEL_SOCKET_ADDRESS;
+
+    return address;
+}
+
+int scim_get_default_socket_timeout ()
+{
+    int timeout = 5000;
+
+    timeout = scim_global_config_read (SCIM_GLOBAL_CONFIG_DEFAULT_SOCKET_TIMEOUT, timeout);
+
+    const char *env = getenv ("SCIM_SOCKET_TIMEOUT");
+
+    if (env && strlen (env))
+        timeout = atoi (env);
+
+    if (timeout <= 0) timeout = -1;
+
+    return timeout;
+}
+
 } // namespace scim
 
 /*
