@@ -23,12 +23,13 @@
  * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  * Boston, MA  02111-1307  USA
  *
- * $Id: scim_socket_transaction.cpp,v 1.17 2003/12/31 05:09:25 suzhe Exp $
+ * $Id: scim_socket_transaction.cpp,v 1.26.2.1 2004/10/03 13:16:12 suzhe Exp $
  *
  */
 
 #define Uses_SCIM_SOCKET_TRANSACTION
 #define Uses_STL_ALGORITHM
+#define Uses_C_STDLIB
 
 #include "scim_private.h"
 #include "scim.h"
@@ -56,7 +57,7 @@ struct SocketTransaction::SocketTransactionImpl
     }
 
     ~SocketTransactionImpl () {
-        delete m_buffer;
+        delete [] m_buffer;
     }
 
     void request_buffer_size (size_t request) {
@@ -66,7 +67,7 @@ struct SocketTransaction::SocketTransactionImpl
 
             memcpy (tmp, m_buffer, m_buffer_size);
             std::swap (tmp, m_buffer);
-            delete tmp;
+            delete [] tmp;
 
             m_buffer_size = bufsize;
         }
@@ -259,11 +260,11 @@ SocketTransaction::put_data (const KeyEvent &key)
 void
 SocketTransaction::put_data (const AttributeList &attrs)
 {
-    size_t size = attrs.size () * (sizeof (unsigned int) * 2 + 1) + sizeof (uint32);
+    size_t size = attrs.size () * (sizeof (unsigned int) * 3 + 1) + sizeof (uint32);
 
     m_impl->request_buffer_size (size);
 
-    m_impl->m_buffer [m_impl->m_write_pos ++] = (unsigned char) SCIM_TRANS_DATA_ATTRLIST;
+    m_impl->m_buffer [m_impl->m_write_pos ++] = (unsigned char) SCIM_TRANS_DATA_ATTRIBUTE_LIST;
 
     scim_uint32tobytes (m_impl->m_buffer + m_impl->m_write_pos, attrs.size ());
 
@@ -271,6 +272,8 @@ SocketTransaction::put_data (const AttributeList &attrs)
 
     for (size_t i=0; i<attrs.size (); i++) {
         m_impl->m_buffer [m_impl->m_write_pos ++] = (unsigned char) attrs[i].get_type ();
+        scim_uint32tobytes (m_impl->m_buffer + m_impl->m_write_pos, attrs[i].get_value ());
+        m_impl->m_write_pos += sizeof (uint32);
         scim_uint32tobytes (m_impl->m_buffer + m_impl->m_write_pos, attrs[i].get_start ());
         m_impl->m_write_pos += sizeof (uint32);
         scim_uint32tobytes (m_impl->m_buffer + m_impl->m_write_pos, attrs[i].get_length ());
@@ -278,6 +281,43 @@ SocketTransaction::put_data (const AttributeList &attrs)
     }
 }
 
+void
+SocketTransaction::put_data (const Property &property)
+{
+    size_t request = property.get_key ().length () +
+                     property.get_label ().length () +
+                     property.get_icon ().length () +
+                     property.get_tip ().length () +
+                     sizeof (uint32) * 4 + 2;
+
+    m_impl->request_buffer_size (request);
+
+    m_impl->m_buffer [m_impl->m_write_pos ++] = (unsigned char) SCIM_TRANS_DATA_PROPERTY;
+
+    put_data (property.get_key ());
+    put_data (property.get_label ());
+    put_data (property.get_icon ());
+    put_data (property.get_tip ());
+
+    m_impl->m_buffer [m_impl->m_write_pos ++] = (unsigned char) property.visible ();
+    m_impl->m_buffer [m_impl->m_write_pos ++] = (unsigned char) property.active ();
+}
+
+void
+SocketTransaction::put_data (const PropertyList &properties)
+{
+    m_impl->request_buffer_size (sizeof(uint32));
+
+    m_impl->m_buffer [m_impl->m_write_pos ++] = (unsigned char) SCIM_TRANS_DATA_PROPERTY_LIST;
+
+    scim_uint32tobytes (m_impl->m_buffer + m_impl->m_write_pos, properties.size ());
+
+    m_impl->m_write_pos += sizeof (uint32);
+
+    for (PropertyList::const_iterator it = properties.begin (); it != properties.end (); ++ it)
+        put_data (*it);
+}
+
 void
 SocketTransaction::put_data (const LookupTable &table)
 {
@@ -292,7 +332,7 @@ SocketTransaction::put_data (const LookupTable &table)
 
     //Can be page down.
     if (table.get_current_page_start () + table.get_current_page_size () <
-        table.number_of_entries ())
+        table.number_of_candidates ())
         stat |= 2;
 
     //Cursor is visible.
@@ -302,16 +342,17 @@ SocketTransaction::put_data (const LookupTable &table)
     m_impl->m_buffer [m_impl->m_write_pos ++] = (unsigned char) SCIM_TRANS_DATA_LOOKUP_TABLE;
     m_impl->m_buffer [m_impl->m_write_pos ++] = stat;
     m_impl->m_buffer [m_impl->m_write_pos ++] = (unsigned char) table.get_current_page_size ();
-    m_impl->m_buffer [m_impl->m_write_pos ++] = (unsigned char) table.get_cursor_pos_in_page ();
+    m_impl->m_buffer [m_impl->m_write_pos ++] = (unsigned char) table.get_cursor_pos_in_current_page ();
 
-    put_data (table.get_page_up_key ());
-    put_data (table.get_page_down_key ());
+    // Store page labels.
+    for (i = 0; i < table.get_current_page_size (); ++i)
+        put_data (table.get_candidate_label (i));
 
-    for (i = 0; i<table.get_current_page_size (); i++)
-        put_data (table.get_page_index_key (i));
-
-    for (i = 0; i<table.get_current_page_size (); i++)
-        put_data (table.get_content_in_page (i));
+    // Store page candidates and attributes.
+    for (i = 0; i < table.get_current_page_size (); ++i) {
+        put_data (table.get_candidate_in_current_page (i));
+        put_data (table.get_attributes_in_current_page (i));
+    }
 }
 
 void
@@ -533,10 +574,11 @@ bool
 SocketTransaction::get_data (AttributeList &attrs)
 {
     if (m_impl->m_write_pos > m_impl->m_read_pos &&
-        m_impl->m_buffer [m_impl->m_read_pos] == SCIM_TRANS_DATA_ATTRLIST) {
+        m_impl->m_buffer [m_impl->m_read_pos] == SCIM_TRANS_DATA_ATTRIBUTE_LIST) {
 
         AttributeType type;
         size_t num;
+        uint32 value;
         uint32 start;
         uint32 length;
 
@@ -550,7 +592,7 @@ SocketTransaction::get_data (AttributeList &attrs)
         num = scim_bytestouint32 (m_impl->m_buffer + m_impl->m_read_pos);
         m_impl->m_read_pos += sizeof (uint32);
 
-        if (m_impl->m_write_pos < (m_impl->m_read_pos + (sizeof (uint32) * 2 + 1) * num)) {
+        if (m_impl->m_write_pos < (m_impl->m_read_pos + (sizeof (uint32) * 3 + 1) * num)) {
             m_impl->m_read_pos -= (sizeof (uint32) + 1);
             return false;
         }
@@ -558,18 +600,108 @@ SocketTransaction::get_data (AttributeList &attrs)
         for (size_t i=0; i<num; i++) {
             type = (AttributeType) m_impl->m_buffer [m_impl->m_read_pos];
             m_impl->m_read_pos ++;
+            value = scim_bytestouint32 (m_impl->m_buffer + m_impl->m_read_pos);
+            m_impl->m_read_pos += sizeof (uint32);
             start = scim_bytestouint32 (m_impl->m_buffer + m_impl->m_read_pos);
             m_impl->m_read_pos += sizeof (uint32);
             length = scim_bytestouint32 (m_impl->m_buffer + m_impl->m_read_pos);
             m_impl->m_read_pos += sizeof (uint32);
 
-            attrs.push_back (Attribute (start, length, type));
+            attrs.push_back (Attribute (start, length, type, value));
         }
         return true;
     }
     return false;
 }
 
+bool
+SocketTransaction::get_data (Property &property)
+{
+    if (m_impl->m_write_pos > m_impl->m_read_pos &&
+        m_impl->m_buffer [m_impl->m_read_pos] == SCIM_TRANS_DATA_PROPERTY) {
+
+        size_t old_read_pos = m_impl->m_read_pos;
+
+        if (m_impl->m_write_pos < (m_impl->m_read_pos + sizeof (uint32) * 4 + 3))
+            return false;
+
+        m_impl->m_read_pos ++;
+
+        String str;
+
+        if (! get_data (str)) {
+            m_impl->m_read_pos = old_read_pos;
+            return false;
+        }
+        property.set_key (str);
+
+        if (! get_data (str)) {
+            m_impl->m_read_pos = old_read_pos;
+            return false;
+        }
+        property.set_label (str);
+
+        if (! get_data (str)) {
+            m_impl->m_read_pos = old_read_pos;
+            return false;
+        }
+        property.set_icon (str);
+
+        if (! get_data (str)) {
+            m_impl->m_read_pos = old_read_pos;
+            return false;
+        }
+        property.set_tip (str);
+
+        if (m_impl->m_write_pos < (m_impl->m_read_pos + 2)) {
+            m_impl->m_read_pos = old_read_pos;
+            return false;
+        }
+
+        property.show ((bool) m_impl->m_buffer [m_impl->m_read_pos]);
+        m_impl->m_read_pos ++;
+
+        property.set_active ((bool) m_impl->m_buffer [m_impl->m_read_pos]);
+        m_impl->m_read_pos ++;
+
+        return true;
+    }
+    return false;
+}
+
+bool
+SocketTransaction::get_data (PropertyList &properties)
+{
+    if (m_impl->m_write_pos > m_impl->m_read_pos &&
+        m_impl->m_buffer [m_impl->m_read_pos] == SCIM_TRANS_DATA_PROPERTY_LIST) {
+
+        size_t old_read_pos = m_impl->m_read_pos;
+        size_t num;
+
+        if (m_impl->m_write_pos < (m_impl->m_read_pos + sizeof (uint32) + 1))
+            return false;
+
+        m_impl->m_read_pos ++;
+
+        num = scim_bytestouint32 (m_impl->m_buffer + m_impl->m_read_pos);
+        m_impl->m_read_pos += sizeof (uint32);
+
+        properties.clear ();
+        Property prop;
+
+        for (size_t i = 0; i < num; ++ i) {
+            if (!get_data (prop)) {
+                m_impl->m_read_pos = old_read_pos;
+                return false;
+            }
+            properties.push_back (prop);
+        }
+
+        return true;
+    }
+    return false;
+}
+
 bool
 SocketTransaction::get_data (CommonLookupTable &table)
 {
@@ -584,11 +716,10 @@ SocketTransaction::get_data (CommonLookupTable &table)
         uint32 page_size;
         uint32 cursor_pos;
 
-        WideString entry;
+        WideString    wstr;
+        AttributeList attrs;
 
-        std::vector<KeyEvent> page_index_keys;
-        KeyEvent page_up_key;
-        KeyEvent page_down_key;
+        std::vector<WideString> labels;
 
         if (m_impl->m_write_pos < (m_impl->m_read_pos + 4))
             return false;
@@ -612,40 +743,37 @@ SocketTransaction::get_data (CommonLookupTable &table)
             return false;
         }
 
-        if (!get_data (page_up_key) ||
-            !get_data (page_down_key)) {
-            m_impl->m_read_pos = old_read_pos;
-            return false;
-        }
-
-        table.set_page_updown_keys (page_up_key, page_down_key);
         table.set_page_size (page_size);
 
-        for (i=0; i<page_size; i++) {
-            if (!get_data (page_up_key)) {
+        for (i = 0; i < page_size; ++i) {
+            if (!get_data (wstr)) {
                 m_impl->m_read_pos = old_read_pos;
                 return false;
             }
-            page_index_keys.push_back (page_up_key);
+            labels.push_back (wstr);
         }
 
-        table.set_page_index_keys (page_index_keys);
+        table.set_candidate_labels (labels);
 
         //Can be paged up.
         if (stat & 1)
-            table.append_entry (0x3400);
+            table.append_candidate (0x3400);
 
-        for (i=0; i<page_size; i++) {
-            if (!get_data (entry)) {
+        for (i = 0; i < page_size; ++i) {
+            if (!get_data (wstr)) {
                 m_impl->m_read_pos = old_read_pos;
                 return false;
             }
-            table.append_entry (entry);
+            if (!get_data (attrs)) {
+                m_impl->m_read_pos = old_read_pos;
+                return false;
+            }
+            table.append_candidate (wstr, attrs);
         }
 
         // Can be paged down.
         if (stat & 2)
-            table.append_entry (0x3400);
+            table.append_candidate (0x3400);
 
         if (stat & 1) {
             table.set_page_size (1);
@@ -653,8 +781,7 @@ SocketTransaction::get_data (CommonLookupTable &table)
             table.set_page_size (page_size);
         }
 
-        for (i=0; i<cursor_pos; i++)
-            table.cursor_down ();
+        table.set_cursor_pos_in_current_page (cursor_pos);
 
         if (stat & 4)
             table.show_cursor (true);
@@ -833,11 +960,21 @@ SocketTransaction::skip_data ()
                 KeyEvent key;
                 return get_data (key);
             }
-            case SCIM_TRANS_DATA_ATTRLIST:
+            case SCIM_TRANS_DATA_ATTRIBUTE_LIST:
             {
                 AttributeList attrs;
                 return get_data (attrs);
             }
+            case SCIM_TRANS_DATA_PROPERTY:
+            {
+                Property prop;
+                return get_data (prop);
+            }
+            case SCIM_TRANS_DATA_PROPERTY_LIST:
+            {
+                PropertyList proplist;
+                return get_data (proplist);
+            }
             case SCIM_TRANS_DATA_LOOKUP_TABLE:
             {
                 CommonLookupTable table;
@@ -881,6 +1018,96 @@ SocketTransaction::clear ()
     m_impl->m_read_pos = SCIM_TRANS_HEADER_SIZE;
 }
 
+static bool
+scim_socket_trans_check_type (const String &types,
+                              const String &atype)
+{
+    std::vector <String> type_list;
+    scim_split_string_list (type_list, types, ',');
+
+    return std::find (type_list.begin (), type_list.end (), atype) != type_list.end ();
+}
+
+bool
+scim_socket_trans_open_connection   (uint32       &key,
+                                     const String &client_type,
+                                     const String &server_type,
+                                     const Socket &socket,
+                                     int           timeout)
+{
+    if (!socket.valid () || !client_type.length () || !server_type.length ())
+        return false;
+
+    SocketTransaction trans;
+
+    trans.put_command (SCIM_TRANS_CMD_REQUEST);
+    trans.put_command (SCIM_TRANS_CMD_OPEN_CONNECTION);
+    trans.put_data (String (SCIM_BINARY_VERSION));
+    trans.put_data (client_type);
+
+    if (trans.write_to_socket (socket)) {
+        int cmd;
+        String server_types;
+        if (trans.read_from_socket (socket, timeout) &&
+            trans.get_command (cmd) && cmd == SCIM_TRANS_CMD_REPLY &&
+            trans.get_data (server_types) && scim_socket_trans_check_type (server_types, server_type) &&
+            trans.get_data (key)) {
+            trans.clear ();
+            trans.put_command (SCIM_TRANS_CMD_REPLY);
+            trans.put_command (SCIM_TRANS_CMD_OK);
+            if (trans.write_to_socket (socket))
+                return true;
+        } else {
+            trans.clear ();
+            trans.put_command (SCIM_TRANS_CMD_REPLY);
+            trans.put_command (SCIM_TRANS_CMD_FAIL);
+            trans.write_to_socket (socket);
+        }
+    }
+
+    return false;
+}
+
+String
+scim_socket_trans_accept_connection (uint32       &key,
+                                     const String &server_types,
+                                     const String &client_types,
+                                     const Socket &socket,
+                                     int           timeout)
+{
+    if (!socket.valid () || !client_types.length () || !server_types.length ())
+        return String ("");
+
+    SocketTransaction trans;
+
+    if (trans.read_from_socket (socket, timeout)) {
+        int cmd;
+        String version;
+        String client_type;
+        if (trans.get_command (cmd)  && cmd == SCIM_TRANS_CMD_REQUEST &&
+            trans.get_command (cmd)  && cmd == SCIM_TRANS_CMD_OPEN_CONNECTION &&
+            trans.get_data (version) && version == String (SCIM_BINARY_VERSION) &&
+            trans.get_data (client_type) && 
+            (scim_socket_trans_check_type (client_types, client_type) || client_type == "ConnectionTester")) {
+            key = (uint32) rand ();
+            trans.clear ();
+            trans.put_command (SCIM_TRANS_CMD_REPLY);
+            trans.put_data (server_types);
+            trans.put_data (key);
+
+            if (trans.write_to_socket (socket) &&
+                trans.read_from_socket (socket, timeout) &&
+                trans.get_command (cmd) && cmd == SCIM_TRANS_CMD_REPLY &&
+                trans.get_command (cmd) && cmd == SCIM_TRANS_CMD_OK) {
+
+                // Client is ok, return the client type.
+                return (client_type == "ConnectionTester") ? String ("") : client_type;
+            }
+        }
+    }
+    return String ("");
+}
+
 } // namespace scim
 
 /*
