@@ -17,6 +17,11 @@ enum {
     LAST_SIGNAL
 };
 
+struct KeyGrabData {
+    std::vector <int> codes;
+    KeyEvent key;
+};
+
 /* GObject, GtkObject methods
  */
 
@@ -33,242 +38,19 @@ static void scim_key_selection_del_key_button_callback (GtkButton             *b
 static void scim_key_selection_list_changed_callback   (GtkTreeSelection      *selection,
                                                         ScimKeySelection      *keyselection);
 
-static KeyCode all_key_codes [] =
-{
-    /* Latin 1 */
-    SCIM_KEY_space,
-    SCIM_KEY_exclam,
-    SCIM_KEY_quotedbl,
-    SCIM_KEY_numbersign,
-    SCIM_KEY_dollar,
-    SCIM_KEY_percent,
-    SCIM_KEY_ampersand,
-    SCIM_KEY_apostrophe,
-    SCIM_KEY_parenleft,
-    SCIM_KEY_parenright,
-    SCIM_KEY_asterisk,
-    SCIM_KEY_plus,
-    SCIM_KEY_comma,
-    SCIM_KEY_minus,
-    SCIM_KEY_period,
-    SCIM_KEY_slash,
-    SCIM_KEY_0,
-    SCIM_KEY_1,
-    SCIM_KEY_2,
-    SCIM_KEY_3,
-    SCIM_KEY_4,
-    SCIM_KEY_5,
-    SCIM_KEY_6,
-    SCIM_KEY_7,
-    SCIM_KEY_8,
-    SCIM_KEY_9,
-    SCIM_KEY_colon,
-    SCIM_KEY_semicolon,
-    SCIM_KEY_less,
-    SCIM_KEY_equal,
-    SCIM_KEY_greater,
-    SCIM_KEY_question,
-    SCIM_KEY_at,
-    SCIM_KEY_A,
-    SCIM_KEY_B,
-    SCIM_KEY_C,
-    SCIM_KEY_D,
-    SCIM_KEY_E,
-    SCIM_KEY_F,
-    SCIM_KEY_G,
-    SCIM_KEY_H,
-    SCIM_KEY_I,
-    SCIM_KEY_J,
-    SCIM_KEY_K,
-    SCIM_KEY_L,
-    SCIM_KEY_M,
-    SCIM_KEY_N,
-    SCIM_KEY_O,
-    SCIM_KEY_P,
-    SCIM_KEY_Q,
-    SCIM_KEY_R,
-    SCIM_KEY_S,
-    SCIM_KEY_T,
-    SCIM_KEY_U,
-    SCIM_KEY_V,
-    SCIM_KEY_W,
-    SCIM_KEY_X,
-    SCIM_KEY_Y,
-    SCIM_KEY_Z,
-    SCIM_KEY_bracketleft,
-    SCIM_KEY_backslash,
-    SCIM_KEY_bracketright,
-    SCIM_KEY_asciicircum,
-    SCIM_KEY_underscore,
-    SCIM_KEY_grave,
-    SCIM_KEY_a,
-    SCIM_KEY_b,
-    SCIM_KEY_c,
-    SCIM_KEY_d,
-    SCIM_KEY_e,
-    SCIM_KEY_f,
-    SCIM_KEY_g,
-    SCIM_KEY_h,
-    SCIM_KEY_i,
-    SCIM_KEY_j,
-    SCIM_KEY_k,
-    SCIM_KEY_l,
-    SCIM_KEY_m,
-    SCIM_KEY_n,
-    SCIM_KEY_o,
-    SCIM_KEY_p,
-    SCIM_KEY_q,
-    SCIM_KEY_r,
-    SCIM_KEY_s,
-    SCIM_KEY_t,
-    SCIM_KEY_u,
-    SCIM_KEY_v,
-    SCIM_KEY_w,
-    SCIM_KEY_x,
-    SCIM_KEY_y,
-    SCIM_KEY_z,
-    SCIM_KEY_braceleft,
-    SCIM_KEY_bar,
-    SCIM_KEY_braceright,
-    SCIM_KEY_asciitilde,
+static void scim_key_grab_button_callback              (GtkButton             *button,
+                                                        ScimKeySelection      *keyselection);
 
-    /* function keys */
-    SCIM_KEY_BackSpace,
-    SCIM_KEY_Tab,
-    SCIM_KEY_Linefeed,
-    SCIM_KEY_Clear,
-    SCIM_KEY_Return,
-    SCIM_KEY_Pause,
-    SCIM_KEY_Scroll_Lock,
-    SCIM_KEY_Sys_Req,
-    SCIM_KEY_Escape,
-    SCIM_KEY_Delete,
+static gboolean scim_key_grab_press_callback           (GtkDialog             *dialog,
+                                                        GdkEventKey           *event,
+                                                        KeyGrabData           *data);
 
-    /* Cursor control & motion */
-    SCIM_KEY_Home,
-    SCIM_KEY_Left,
-    SCIM_KEY_Up,
-    SCIM_KEY_Right,
-    SCIM_KEY_Down,
-    SCIM_KEY_Page_Up,
-    SCIM_KEY_Page_Down,
-    SCIM_KEY_End,
-    SCIM_KEY_Begin,
+static gboolean scim_key_grab_release_callback         (GtkDialog             *dialog,
+                                                        GdkEventKey           *event,
+                                                        KeyGrabData           *data);
 
-    /* Auxilliary Functions */
-    SCIM_KEY_F1,
-    SCIM_KEY_F2,
-    SCIM_KEY_F3,
-    SCIM_KEY_F4,
-    SCIM_KEY_F5,
-    SCIM_KEY_F6,
-    SCIM_KEY_F7,
-    SCIM_KEY_F8,
-    SCIM_KEY_F9,
-    SCIM_KEY_F10,
-    SCIM_KEY_F11,
-    SCIM_KEY_F12,
-
-    SCIM_KEY_Shift_L,
-    SCIM_KEY_Shift_R,
-    SCIM_KEY_Control_L,
-    SCIM_KEY_Control_R,
-    SCIM_KEY_Caps_Lock,
-    SCIM_KEY_Shift_Lock,
-
-    SCIM_KEY_Meta_L,
-    SCIM_KEY_Meta_R,
-    SCIM_KEY_Alt_L,
-    SCIM_KEY_Alt_R,
-    SCIM_KEY_Super_L,
-    SCIM_KEY_Super_R,
-    SCIM_KEY_Hyper_L,
-    SCIM_KEY_Hyper_R,
-
-    /* International & multi-key character composition */
-    SCIM_KEY_Multi_key,
-    SCIM_KEY_Codeinput,
-    SCIM_KEY_SingleCandidate,
-    SCIM_KEY_MultipleCandidate,
-    SCIM_KEY_PreviousCandidate,
-
-    /* Japanese keyboard support */
-    SCIM_KEY_Kanji,
-    SCIM_KEY_Muhenkan,
-    SCIM_KEY_Henkan_Mode,
-    SCIM_KEY_Henkan,
-    SCIM_KEY_Romaji,
-    SCIM_KEY_Hiragana,
-    SCIM_KEY_Katakana,
-    SCIM_KEY_Hiragana_Katakana,
-    SCIM_KEY_Zenkaku,
-    SCIM_KEY_Hankaku,
-    SCIM_KEY_Zenkaku_Hankaku,
-    SCIM_KEY_Touroku,
-    SCIM_KEY_Massyo,
-    SCIM_KEY_Kana_Lock,
-    SCIM_KEY_Kana_Shift,
-    SCIM_KEY_Eisu_Shift,
-    SCIM_KEY_Eisu_toggle,
-    SCIM_KEY_Kanji_Bangou,
-    SCIM_KEY_Zen_Koho,
-    SCIM_KEY_Mae_Koho,
-
-    /* Misc Functions */
-    SCIM_KEY_Select,
-    SCIM_KEY_Print,
-    SCIM_KEY_Execute,
-    SCIM_KEY_Insert,
-    SCIM_KEY_Undo,
-    SCIM_KEY_Redo,
-    SCIM_KEY_Menu,
-    SCIM_KEY_Find,
-    SCIM_KEY_Cancel,
-    SCIM_KEY_Help,
-    SCIM_KEY_Break,
-    SCIM_KEY_Mode_switch,
-    SCIM_KEY_Num_Lock,
-
-    /* keypad */
-    SCIM_KEY_KP_Space,
-    SCIM_KEY_KP_Tab,
-    SCIM_KEY_KP_Enter,
-    SCIM_KEY_KP_F1,
-    SCIM_KEY_KP_F2,
-    SCIM_KEY_KP_F3,
-    SCIM_KEY_KP_F4,
-    SCIM_KEY_KP_Home,
-    SCIM_KEY_KP_Left,
-    SCIM_KEY_KP_Up,
-    SCIM_KEY_KP_Right,
-    SCIM_KEY_KP_Down,
-    SCIM_KEY_KP_Page_Up,
-    SCIM_KEY_KP_Page_Down,
-    SCIM_KEY_KP_End,
-    SCIM_KEY_KP_Begin,
-    SCIM_KEY_KP_Insert,
-    SCIM_KEY_KP_Delete,
-    SCIM_KEY_KP_Equal,
-    SCIM_KEY_KP_Multiply,
-    SCIM_KEY_KP_Add,
-    SCIM_KEY_KP_Separator,
-    SCIM_KEY_KP_Subtract,
-    SCIM_KEY_KP_Decimal,
-    SCIM_KEY_KP_Divide,
-
-    SCIM_KEY_KP_0,
-    SCIM_KEY_KP_1,
-    SCIM_KEY_KP_2,
-    SCIM_KEY_KP_3,
-    SCIM_KEY_KP_4,
-    SCIM_KEY_KP_5,
-    SCIM_KEY_KP_6,
-    SCIM_KEY_KP_7,
-    SCIM_KEY_KP_8,
-    SCIM_KEY_KP_9,
-
-    SCIM_KEY_NullKey
-};
+static void scim_key_selection_set_key_event           (ScimKeySelection      *keyselection,
+                                                        KeyEvent               event);
 
 static GtkWidgetClass *parent_class = NULL;
 
@@ -344,37 +126,11 @@ scim_key_selection_class_init (ScimKeySelectionClass *klass)
     klass->changed = NULL;
 }
 
-
-static GList *
-create_key_names_list ()
-{
-    GList *list = NULL;
-    String name;
-
-    for (int i = 0; all_key_codes [i] != SCIM_KEY_NullKey; ++ i) {
-        if (scim_key_to_string (name, KeyEvent (all_key_codes [i], 0))) {
-            list = g_list_append (list, g_strdup (name.c_str ()));
-        }
-    }
-
-    return list;
-}
-
-static void
-destroy_key_names_list (GList *list)
-{
-    for (GList *ptr = list; ptr != NULL; ptr = g_list_next (ptr))
-        g_free (ptr->data);
-
-    g_list_free (list);
-}
-
 static void
 scim_key_selection_init (ScimKeySelection *keyselection)
 {
     GtkWidget *table;
-    GtkWidget *hbox1;
-    GtkWidget *hbox2;
+    GtkWidget *hbox;
     GtkWidget *frame;
     GtkWidget *label;
 
@@ -384,8 +140,6 @@ scim_key_selection_init (ScimKeySelection *keyselection)
     GtkCellRenderer *list_cell;
     GtkTreeViewColumn *list_column;
 
-    GList     *key_list;
-
     frame = gtk_frame_new (NULL);
     gtk_widget_show (frame);
     gtk_box_pack_start (GTK_BOX (keyselection), frame, TRUE, TRUE, 0);
@@ -443,14 +197,21 @@ scim_key_selection_init (ScimKeySelection *keyselection)
                       (GtkAttachOptions) (0), 4, 4);
     gtk_misc_set_alignment (GTK_MISC (label), 1, 0.5);
 
-    keyselection->combo = gtk_combo_new ();
-    gtk_widget_show (keyselection->combo);
-    gtk_table_attach (GTK_TABLE (table), keyselection->combo, 1, 2, 0, 1,
+    hbox = gtk_hbox_new (FALSE, 0);
+    gtk_widget_show (hbox);
+    gtk_table_attach (GTK_TABLE (table), hbox, 1, 2, 0, 1,
                       (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
                       (GtkAttachOptions) (0), 4, 4);
-    gtk_combo_set_value_in_list (GTK_COMBO (keyselection->combo), TRUE, FALSE);
-    gtk_combo_set_case_sensitive (GTK_COMBO (keyselection->combo), TRUE);
-    gtk_entry_set_editable (GTK_ENTRY (GTK_COMBO (keyselection->combo)->entry), FALSE);
+
+    keyselection->key_code = gtk_entry_new ();
+    gtk_widget_show (keyselection->key_code);
+    gtk_box_pack_start (GTK_BOX (hbox), keyselection->key_code, TRUE, TRUE, 2);
+    gtk_entry_set_editable (GTK_ENTRY (keyselection->key_code), FALSE);
+
+    button = gtk_button_new_with_label (_("..."));
+    gtk_widget_show (button);
+    gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 2);
+    g_signal_connect (G_OBJECT (button), "clicked", G_CALLBACK (scim_key_grab_button_callback), keyselection);
 
     label = gtk_label_new (_("Modifiers:"));
     gtk_widget_show (label);
@@ -459,51 +220,47 @@ scim_key_selection_init (ScimKeySelection *keyselection)
                       (GtkAttachOptions) (0), 4, 4);
     gtk_misc_set_alignment (GTK_MISC (label), 1, 0.5);
 
-    hbox1 = gtk_hbox_new (FALSE, 0);
-    gtk_widget_show (hbox1);
-    gtk_table_attach (GTK_TABLE (table), hbox1, 1, 2, 1, 2,
+    hbox = gtk_hbox_new (FALSE, 0);
+    gtk_widget_show (hbox);
+    gtk_table_attach (GTK_TABLE (table), hbox, 1, 2, 1, 2,
                       (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
                       (GtkAttachOptions) (0), 4, 4);
 
     keyselection->toggle_ctrl = gtk_check_button_new_with_mnemonic (_("_Ctrl")); 
     gtk_widget_show (keyselection->toggle_ctrl);
-    gtk_box_pack_start (GTK_BOX (hbox1), keyselection->toggle_ctrl, TRUE, TRUE, 2);
+    gtk_box_pack_start (GTK_BOX (hbox), keyselection->toggle_ctrl, TRUE, TRUE, 2);
 
     keyselection->toggle_alt = gtk_check_button_new_with_mnemonic (_("A_lt")); 
     gtk_widget_show (keyselection->toggle_alt);
-    gtk_box_pack_start (GTK_BOX (hbox1), keyselection->toggle_alt, TRUE, TRUE, 2);
+    gtk_box_pack_start (GTK_BOX (hbox), keyselection->toggle_alt, TRUE, TRUE, 2);
 
     keyselection->toggle_shift = gtk_check_button_new_with_mnemonic (_("_Shift")); 
     gtk_widget_show (keyselection->toggle_shift);
-    gtk_box_pack_start (GTK_BOX (hbox1), keyselection->toggle_shift, TRUE, TRUE, 2);
+    gtk_box_pack_start (GTK_BOX (hbox), keyselection->toggle_shift, TRUE, TRUE, 2);
 
     keyselection->toggle_release = gtk_check_button_new_with_mnemonic (_("_Release")); 
     gtk_widget_show (keyselection->toggle_release);
-    gtk_box_pack_start (GTK_BOX (hbox1), keyselection->toggle_release, TRUE, TRUE, 2);
+    gtk_box_pack_start (GTK_BOX (hbox), keyselection->toggle_release, TRUE, TRUE, 2);
 
-    hbox2 = gtk_hbox_new (TRUE, 0);
-    gtk_widget_show (hbox2);
-    gtk_box_pack_start (GTK_BOX (keyselection), hbox2, FALSE, FALSE, 4);
+    hbox = gtk_hbox_new (TRUE, 0);
+    gtk_widget_show (hbox);
+    gtk_box_pack_start (GTK_BOX (keyselection), hbox, FALSE, FALSE, 4);
 
     button = gtk_button_new_from_stock ("gtk-add");
     gtk_widget_show (button);
-    gtk_box_pack_start (GTK_BOX (hbox2), button, TRUE, TRUE, 4);
+    gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 4);
     g_signal_connect ((gpointer) button, "clicked",
                       G_CALLBACK (scim_key_selection_add_key_button_callback),
                       keyselection);
 
     button = gtk_button_new_from_stock ("gtk-delete");
     gtk_widget_show (button);
-    gtk_box_pack_start (GTK_BOX (hbox2), button, TRUE, TRUE, 4);
+    gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 4);
     g_signal_connect ((gpointer) button, "clicked",
                       G_CALLBACK (scim_key_selection_del_key_button_callback),
                       keyselection);
 
     // Create key names list
-    key_list = create_key_names_list ();
-    gtk_combo_set_popdown_strings (GTK_COMBO (keyselection->combo), key_list);
-    destroy_key_names_list (key_list);
-
     keyselection->keys = NULL;
 }
 
@@ -531,7 +288,7 @@ scim_key_selection_add_key_button_callback (GtkButton        *button,
     if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (keyselection->toggle_shift)))
         key += String ("Shift+");
 
-    key += String (gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (keyselection->combo)->entry)));
+    key += String (gtk_entry_get_text (GTK_ENTRY (keyselection->key_code)));
 
     if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (keyselection->toggle_release)))
         key += String ("+KeyRelease");
@@ -569,6 +326,83 @@ scim_key_selection_del_key_button_callback (GtkButton       *button,
     }
 }
 
+static void
+scim_key_grab_button_callback (GtkButton        *button,
+                               ScimKeySelection *keyselection)
+{
+    KeyGrabData key_grab_data;
+    GtkWidget *dialog;
+
+    dialog = gtk_message_dialog_new (NULL,
+                                     GTK_DIALOG_MODAL,
+                                     GTK_MESSAGE_INFO,
+                                     GTK_BUTTONS_CANCEL,
+                                     _("Press a key (or a key combination).\n"
+                                       "This dialog will be closed when the key is released."));
+
+    gtk_window_set_title (GTK_WINDOW (dialog), _("Grabbing a key."));
+
+    g_signal_connect (G_OBJECT (dialog), "key-press-event",
+                      G_CALLBACK (scim_key_grab_press_callback), &key_grab_data);
+
+    g_signal_connect (G_OBJECT (dialog), "key-release-event",
+                      G_CALLBACK (scim_key_grab_release_callback), &key_grab_data);
+
+    if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_OK) {
+        scim_key_selection_set_key_event (keyselection, key_grab_data.key);
+    }
+
+    gtk_widget_destroy (dialog);
+}
+
+static KeyEvent
+keyevent_gdk_to_scim (const GdkEventKey &gdkevent)
+{
+    return KeyEvent (gdkevent.keyval,
+                     (gdkevent.type == GDK_KEY_PRESS)?
+                     (gdkevent.state & ~ SCIM_KEY_ReleaseMask):
+                     (gdkevent.state | SCIM_KEY_ReleaseMask));
+}
+
+static gboolean
+scim_key_grab_press_callback (GtkDialog   *dialog,
+                              GdkEventKey *event,
+                              KeyGrabData *data)
+{
+    KeyEvent key = keyevent_gdk_to_scim (*event);
+    data->codes.push_back (key.code);
+
+    return TRUE;
+}
+
+static gboolean
+scim_key_grab_release_callback (GtkDialog   *dialog,
+                                GdkEventKey *event,
+                                KeyGrabData *data)
+{
+    KeyEvent key = keyevent_gdk_to_scim (*event);
+
+    data->key.mask |= (key.mask & (~ SCIM_KEY_ReleaseMask));
+
+    if (data->key.code == 0 || key.code < SCIM_KEY_Shift_L || key.code > SCIM_KEY_Hyper_R)
+        data->key.code = key.code;
+
+    for (std::vector<int>::iterator it = data->codes.begin (); it != data->codes.end (); ++it) {
+        if (*it == key.code) {
+            data->codes.erase (it);
+            break;
+        }
+    }
+
+    if (data->key.code >= SCIM_KEY_Shift_L && data->key.code <= SCIM_KEY_Hyper_R)
+        data->key.mask |= SCIM_KEY_ReleaseMask;
+
+    if (data->codes.size () == 0)
+        gtk_dialog_response (dialog, GTK_RESPONSE_OK);
+
+    return TRUE;
+}
+
 /* public api
  */
 GtkWidget*
@@ -584,16 +418,22 @@ scim_key_selection_append_keys (ScimKeySelection *keyselection,
     g_return_if_fail (SCIM_IS_KEY_SELECTION (keyselection));
     g_return_if_fail (keys != NULL);
 
-    std::vector <String> keylist;
+    KeyEventList keylist;
+
+    if (!scim_string_to_key_list (keylist, keys))
+        return;
+
     GtkTreeIter iter;
     GtkTreeModel *model;
 
-    scim_split_string_list (keylist, keys);
+    String str;
 
     for (int i = 0; i < keylist.size (); ++ i) {
-        gtk_list_store_append (keyselection->list_model, &iter);
-        gtk_list_store_set (keyselection->list_model, &iter,
-                            0, keylist [i].c_str (), -1);
+        if (scim_key_to_string (str, keylist [i])) {
+            gtk_list_store_append (keyselection->list_model, &iter);
+            gtk_list_store_set (keyselection->list_model, &iter,
+                                0, str.c_str (), -1);
+        }
     }
 }
 
@@ -650,28 +490,36 @@ scim_key_selection_list_changed_callback (GtkTreeSelection *selection,
     if (gtk_tree_selection_get_selected (selection, &model, &iter)) {
         gtk_tree_model_get (model, &iter, 0, &keystr, -1);
 
-        if (scim_string_to_key (keyevent, String (keystr))) {
-            gtk_toggle_button_set_active (
-                GTK_TOGGLE_BUTTON (keyselection->toggle_ctrl),
-                keyevent.is_control_down ());
-            gtk_toggle_button_set_active (
-                GTK_TOGGLE_BUTTON (keyselection->toggle_alt),
-                keyevent.is_alt_down ());
-            gtk_toggle_button_set_active (
-                GTK_TOGGLE_BUTTON (keyselection->toggle_shift),
-                keyevent.is_shift_down ());
-            gtk_toggle_button_set_active (
-                GTK_TOGGLE_BUTTON (keyselection->toggle_release),
-                keyevent.is_key_release ());
+        if (scim_string_to_key (keyevent, String (keystr)))
+            scim_key_selection_set_key_event (keyselection, keyevent);
+    }
+}
 
-            keyevent.mask = 0;
-            String str;
+static void
+scim_key_selection_set_key_event (ScimKeySelection *keyselection,
+                                  KeyEvent          event)
+{
+    g_return_if_fail (SCIM_IS_KEY_SELECTION (keyselection));
 
-            if (scim_key_to_string (str, keyevent)) {
-                gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (keyselection->combo)->entry),
-                                    str.c_str ());
-            }
-        }
+    gtk_toggle_button_set_active (
+        GTK_TOGGLE_BUTTON (keyselection->toggle_ctrl),
+        event.is_control_down ());
+    gtk_toggle_button_set_active (
+        GTK_TOGGLE_BUTTON (keyselection->toggle_alt),
+        event.is_alt_down ());
+    gtk_toggle_button_set_active (
+        GTK_TOGGLE_BUTTON (keyselection->toggle_shift),
+        event.is_shift_down ());
+    gtk_toggle_button_set_active (
+        GTK_TOGGLE_BUTTON (keyselection->toggle_release),
+        event.is_key_release ());
+
+    event.mask = 0;
+    String str;
+
+    if (scim_key_to_string (str, event)) {
+        gtk_entry_set_text (GTK_ENTRY (keyselection->key_code),
+                            str.c_str ());
     }
 }
 
